@@ -21,7 +21,12 @@ from PyQt4.QtCore import (Qt, pyqtSignature, SIGNAL, QMimeData, QTimer,
                           QSettings, QCoreApplication, QUrl)
 
 from spyderlib.widgets.internalshell import InternalShell
-from spyderlib.widgets.externalshell.namespacebrowser import NamespaceBrowser
+try:  # Support for spyder < 3
+    from spyderlib.widgets.externalshell.namespacebrowser import \
+         NamespaceBrowser
+except ImportError:
+    from spyderlib.widgets.variableexplorer.namespacebrowser import \
+         NamespaceBrowser
 from spyderlib.widgets.sourcecode.codeeditor import CodeEditor
 from spyderlib.utils.misc import get_error_match
 
@@ -47,8 +52,12 @@ from remote_thread import RemoteThread
 logger = logging.getLogger('spykeviewer')
 
 
-# Monkeypatch variable editor
-from spyderlib.widgets.dicteditor import DictDelegate
+# Monkeypatch variable editor to report error in message box
+try:  # spyder < 3
+    from spyderlib.widgets.dicteditor import DictDelegate
+except ImportError:
+    from spyderlib.widgets.variableexplorer.collectionseditor import \
+         CollectionsDelegate as DictDelegate
 _orig_createEditor = DictDelegate.createEditor
 
 
@@ -524,13 +533,19 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         # Variable browser
         self.browser = NamespaceBrowser(self.variableExplorerDock)
         self.browser.set_shellwidget(self.console)
-        self.browser.setup(
+        setup_args = dict(
             check_all=True, exclude_private=True,
             exclude_uppercase=False, exclude_capitalized=False,
             exclude_unsupported=False, truncate=False, minmax=False,
-            collvalue=False, remote_editing=False, inplace=False,
-            autorefresh=False,
+            remote_editing=False, autorefresh=False,
             excluded_names=excludes)
+
+        # spyder < 2.3.8
+        if 'collvalue' in self.browser.setup.__code__.co_varnames:
+            setup_args['collvalue'] = False
+            setup_args['inplace'] = False
+
+        self.browser.setup(**setup_args)
         self.variableExplorerDock.setWidget(self.browser)
 
         # History
