@@ -2,8 +2,7 @@ from PyQt4.QtGui import (QDockWidget, QMenu, QAction, QMessageBox,
                          QItemSelectionRange, QItemSelection,
                          QStandardItemModel, QStandardItem,
                          QItemSelectionModel)
-from PyQt4.QtCore import Qt, pyqtSignal, QModelIndex
-from spyderlib.widgets.dicteditor import DictEditor
+from PyQt4.QtCore import Qt, pyqtSignal
 from spyderlib.utils.qthelpers import get_icon
 import neo
 
@@ -90,6 +89,51 @@ class NeoNavigationDock(QDockWidget, Ui_neoNavigationDock):
 
         return name[::-1]
 
+    def ensure_not_filtered(self, objects, all_objects, filters):
+        """ Deactivates all filters that prevent the the given sequence
+        of objects to be displayed. The passed filter tuple list is
+        modified to only include valid filters.
+
+        :param sequence objects: The objects that need to be visible.
+        :param sequence all_objects: The whole object list to be filtered,
+            including objects that are allowed to be hidden.
+        :param sequence filters: A sequence of (Filter, name) tuples.
+        """
+        objset = set(objects)
+        if not objset.issubset(
+                set(self.parent.filter_list(all_objects, filters))):
+            i = 1
+            while i <= len(filters):
+                test_filters = filters[:i]
+                if objset.issubset(set(self.parent.filter_list(
+                        all_objects, test_filters))):
+                    i += 1
+                else:
+                    test_filters[-1][0].active = False
+                    filters.pop(i - 1)
+
+        for o in objects:
+            i = 0
+            while i < len(filters):
+                if self.parent.is_filtered(o, [filters[i]]):
+                    filters[i][0].active = False
+                    filters.pop(i)
+                else:
+                    i += 1
+
+    def filter_ordered(self, objects, filters):
+        """ Filter a sequence of objects with a sequence of filters. Apply
+        the filters in the order given by the sequence. Return the filtered
+        list.
+        """
+        for f in filters:
+            if f[0].combined:
+                objects = self.parent.filter_list(objects, [f])
+            else:
+                objects = [o for o in objects
+                           if not self.parent.is_filtered(o, [f])]
+        return objects
+
     def populate_neo_block_list(self):
         """ Fill the block list with appropriate entries.
         Qt.UserRole: The :class:`neo.Block` object
@@ -98,18 +142,17 @@ class NeoNavigationDock(QDockWidget, Ui_neoNavigationDock):
 
         filters = self.parent.get_active_filters('Block')
 
-        blocks = self.parent.filter_list(
+        blocks = self.filter_ordered(
             self.parent.block_names.keys(), filters)
         for b in blocks:
-            if self.parent.is_filtered(b, filters):
-                continue
-
             item = QStandardItem(self.parent.block_names[b])
             item.setData(b, Qt.UserRole)
             self.block_model.appendRow(item)
 
         self.neoBlockList.setCurrentIndex(self.block_model.index(0, 0))
         self.set_blocks_label()
+        if not blocks:
+            self.selected_blocks_changed()
 
     def populate_neo_segment_list(self):
         """ Fill the segment list with appropriate entries.
@@ -117,27 +160,26 @@ class NeoNavigationDock(QDockWidget, Ui_neoNavigationDock):
         """
         self.segment_model.clear()
 
+        segments = []
+        for b in self.blocks():
+            segments.extend(b.segments)
+
         filters = self.parent.get_active_filters('Segment')
+        segments = self.filter_ordered(segments, filters)
+        for i, s in enumerate(segments):
+            if s.name:
+                name = s.name + ' (%s-%i)' % \
+                    (self.parent.block_ids[s.block], i)
+            else:
+                name = '%s-%i' % (self.parent.block_ids[s.block], i)
 
-        for index in self.neoBlockList.selectedIndexes():
-            block = self.block_model.data(index, Qt.UserRole)
-
-            segments = self.parent.filter_list(block.segments, filters)
-            for i, s in enumerate(segments):
-                if self.parent.is_filtered(s, filters):
-                    continue
-
-                if s.name:
-                    name = s.name + ' (%s-%i)' % \
-                        (self.parent.block_ids[s.block], i)
-                else:
-                    name = '%s-%i' % (self.parent.block_ids[s.block], i)
-
-                new_item = QStandardItem(name)
-                new_item.setData(s, Qt.UserRole)
-                self.segment_model.appendRow(new_item)
+            new_item = QStandardItem(name)
+            new_item.setData(s, Qt.UserRole)
+            self.segment_model.appendRow(new_item)
 
         self.neoSegmentList.setCurrentIndex(self.segment_model.index(0, 0))
+        if api.config.autoselect_segments:
+            self.neoSegmentList.selectAll()
         self.selected_segments_changed()
 
     def populate_neo_channel_group_list(self):
@@ -148,33 +190,33 @@ class NeoNavigationDock(QDockWidget, Ui_neoNavigationDock):
         self.channelgroup_model.clear()
         self.parent.channel_group_names.clear()
 
+        rcgs = []
+        for b in self.blocks():
+            rcgs.extend(b.recordingchannelgroups)
+
         filters = self.parent.get_active_filters(
             'Recording Channel Group')
+        rcgs = self.filter_ordered(rcgs, filters)
 
-        for index in self.neoBlockList.selectedIndexes():
-            block = self.block_model.data(index, Qt.UserRole)
-
-            rcgs = self.parent.filter_list(
-                block.recordingchannelgroups, filters)
-            for i, rcg in enumerate(rcgs):
-                if self.parent.is_filtered(rcg, filters):
-                    continue
-
-                self.parent.channel_group_names[rcg] = '%s-%s' % (
-                    self.parent.block_ids[rcg.block],
-                    self.get_letter_id(i, True))
-                if rcg.name:
-                    name = rcg.name + ' (%s)' % \
-                                      self.parent.channel_group_names[rcg]
-                else:
-                    name = self.parent.channel_group_names[rcg]
-                new_item = QStandardItem(name)
-                new_item.setData(rcg, Qt.UserRole)
-                self.channelgroup_model.appendRow(new_item)
+        for i, rcg in enumerate(rcgs):
+            self.parent.channel_group_names[rcg] = '%s-%s' % (
+                self.parent.block_ids[rcg.block],
+                self.get_letter_id(i, True))
+            if rcg.name:
+                name = rcg.name + ' (%s)' % \
+                                  self.parent.channel_group_names[rcg]
+            else:
+                name = self.parent.channel_group_names[rcg]
+            new_item = QStandardItem(name)
+            new_item.setData(rcg, Qt.UserRole)
+            self.channelgroup_model.appendRow(new_item)
 
         self.neoChannelGroupList.setCurrentIndex(
             self.channelgroup_model.index(0, 0))
-        self.set_channel_groups_label()
+        if api.config.autoselect_channel_groups:
+            self.neoChannelGroupList.selectAll()
+        elif not rcgs:
+            self.selected_channel_groups_changed()
 
     def populate_neo_channel_list(self):
         """ Fill the channel list with appropriate entries. Data slots:
@@ -184,35 +226,36 @@ class NeoNavigationDock(QDockWidget, Ui_neoNavigationDock):
         self.channel_model.clear()
         channels = set()
 
-        filters = self.parent.get_active_filters(
-            'Recording Channel')
-
-        for index in self.neoChannelGroupList.selectedIndexes():
-            channel_group = self.channelgroup_model.data(index, Qt.UserRole)
-
-            rcs = self.parent.filter_list(
-                channel_group.recordingchannels, filters)
-            for rc in rcs:
+        rcs = []
+        rc_group_name = {}
+        for rcg in self.recording_channel_groups():
+            for rc in rcg.recordingchannels:
                 if not api.config.duplicate_channels and rc in channels:
                     continue
-                if self.parent.is_filtered(rc, filters):
-                    continue
-
                 channels.add(rc)
-                identifier = '%s.%d' % \
-                             (self.parent.channel_group_names[channel_group],
-                              rc.index)
-                if rc.name:
-                    name = rc.name + ' (%s)' % identifier
-                else:
-                    name = identifier
+                rcs.append(rc)
+                rc_group_name[rc] = self.parent.channel_group_names[rcg]
 
-                new_item = QStandardItem(name)
-                new_item.setData(rc, Qt.UserRole)
-                new_item.setData(rc.index, Qt.UserRole + 1)
-                self.channel_model.appendRow(new_item)
+        filters = self.parent.get_active_filters(
+            'Recording Channel')
+        rcs = self.filter_ordered(rcs, filters)
 
-        self.neoChannelList.selectAll()
+        for rc in rcs:
+            identifier = '%s.%d' % \
+                         (rc_group_name[rc],
+                          rc.index)
+            if rc.name:
+                name = rc.name + ' (%s)' % identifier
+            else:
+                name = identifier
+
+            new_item = QStandardItem(name)
+            new_item.setData(rc, Qt.UserRole)
+            new_item.setData(rc.index, Qt.UserRole + 1)
+            self.channel_model.appendRow(new_item)
+
+        if api.config.autoselect_channels:
+            self.neoChannelList.selectAll()
         self.selected_channels_changed()
 
     def populate_neo_unit_list(self):
@@ -221,24 +264,27 @@ class NeoNavigationDock(QDockWidget, Ui_neoNavigationDock):
         """
         self.unit_model.clear()
 
+        units = []
+        for rcg in self.recording_channel_groups():
+            units.extend(rcg.units)
+
         filters = self.parent.get_active_filters('Unit')
+        units = self.filter_ordered(units, filters)
 
-        for index in self.neoChannelGroupList.selectedIndexes():
-            rcg = self.channelgroup_model.data(index, Qt.UserRole)
-
-            units = self.parent.filter_list(rcg.units, filters)
-            for i, u in enumerate(units):
-                if self.parent.is_filtered(u, filters):
-                    continue
-                if u.name:
-                    name = u.name + ' (%s-%d)' % \
-                           (self.parent.channel_group_names[rcg], i)
-                else:
-                    name = '%s-%d' % (self.parent.channel_group_names[rcg], i)
-                new_item = QStandardItem(name)
-                new_item.setData(u, Qt.UserRole)
-                self.unit_model.appendRow(new_item)
+        for i, u in enumerate(units):
+            if self.parent.is_filtered(u, filters):
+                continue
+            if u.name:
+                name = u.name + ' (%s-%d)' % \
+                       (self.parent.channel_group_names[rcg], i)
+            else:
+                name = '%s-%d' % (self.parent.channel_group_names[rcg], i)
+            new_item = QStandardItem(name)
+            new_item.setData(u, Qt.UserRole)
+            self.unit_model.appendRow(new_item)
 
+        if api.config.autoselect_units:
+            self.neoUnitList.selectAll()
         self.selected_units_changed()
 
     def set_blocks_label(self):
@@ -281,24 +327,7 @@ class NeoNavigationDock(QDockWidget, Ui_neoNavigationDock):
                 self.segment_model.rowCount()))
 
     def _edit_item_annotations(self, index, model):
-        self.edit_annotations(model.data(index, Qt.UserRole))
-
-    def edit_annotations(self, data):
-        """ Edit annotations of a Neo object.
-        """
-        editor = DictEditor(self.parent)
-        title = 'Edit annotations'
-        if data.name:
-            title += ' for %s' % data.name
-        editor.setup(data.annotations, title)
-        editor.accepted.connect(
-            lambda: self._editor_ok(data, editor))
-        editor.show()
-        editor.raise_()
-        editor.activateWindow()
-
-    def _editor_ok(self, data, editor):
-        data.annotations = editor.get_value()
+        api.annotation_editor(model.data(index, Qt.UserRole))
 
     def remove_selected(self, list_widget):
         """ Remove all selected objects from the given list widget.
@@ -419,34 +448,51 @@ class NeoNavigationDock(QDockWidget, Ui_neoNavigationDock):
                 self.neoUnitList.selectedIndexes()]
 
     def set_selection(self, data):
-        """ Set the selected data. All data in the selection has to be loaded
-        before!
+        """ Set the selected data.
         """
         block_list = []
         for b in data['blocks']:
-            loaded = NeoDataProvider.get_block(b[1], b[0], False)
+            cl = None
+            rp = None
+            if len(b) > 2:
+                cl = NeoDataProvider.find_io_class(b[2])
+            if len(b) > 3:
+                rp = b[3]
+            loaded = NeoDataProvider.get_block(
+                b[1], b[0], force_io=cl, read_params=rp)
             if loaded is None:
                 raise IOError('One of the files contained in the '
                               'selection could not be loaded!')
             block_list.append(loaded)
-        rcg_list = [block_list[rcg[1]].recordingchannelgroups[rcg[0]]
-                    for rcg in data['channel_groups']]
 
+        block_set = set([(b[0], b[1]) for b in data['blocks']])
         # Select blocks
+        self.ensure_not_filtered(block_list, self.parent.block_names.keys(),
+                                 self.parent.get_active_filters('Block'))
+        self.populate_neo_block_list()
         selection = QItemSelection()
         for i in self.block_model.findItems(
                 '*', Qt.MatchWrap | Qt.MatchWildcard):
             block = i.data(Qt.UserRole)
-            t = [NeoDataProvider.block_indices[block],
-                 self.parent.block_files[block]]
+            t = (NeoDataProvider.block_indices[block],
+                 self.parent.block_files[block])
 
-            if t in data['blocks']:
+            if t in block_set:
                 selection.append(QItemSelectionRange(
                     self.block_model.indexFromItem(i)))
         self.neoBlockList.selectionModel().select(
             selection, QItemSelectionModel.ClearAndSelect)
 
         # Select segments
+        seg_list = [block_list[idx[1]].segments[idx[0]]
+                    for idx in data['segments']]
+        all_segs = []
+        for b in self.blocks():
+            all_segs.extend(b.segments)
+        self.ensure_not_filtered(seg_list, all_segs,
+                                 self.parent.get_active_filters('Segment'))
+        self.populate_neo_segment_list()
+
         selection = QItemSelection()
         for i in self.segment_model.findItems(
                 '*', Qt.MatchWrap | Qt.MatchWildcard):
@@ -463,6 +509,16 @@ class NeoNavigationDock(QDockWidget, Ui_neoNavigationDock):
             selection, QItemSelectionModel.ClearAndSelect)
 
         # Select recording channel groups
+        rcg_list = [block_list[rcg[1]].recordingchannelgroups[rcg[0]]
+                    for rcg in data['channel_groups']]
+        all_rcgs = []
+        for b in self.blocks():
+            all_rcgs.extend(b.recordingchannelgroups)
+        self.ensure_not_filtered(
+            rcg_list, all_rcgs,
+            self.parent.get_active_filters('Recording Channel Group'))
+        self.populate_neo_channel_group_list()
+
         selection = QItemSelection()
         for i in self.channelgroup_model.findItems(
                 '*', Qt.MatchWrap | Qt.MatchWildcard):
@@ -479,6 +535,19 @@ class NeoNavigationDock(QDockWidget, Ui_neoNavigationDock):
             selection, QItemSelectionModel.ClearAndSelect)
 
         # Select channels
+        rc_list = [rcg_list[rc[1]].recordingchannels[rc[0]]
+                   for rc in data['channels']]
+        all_rcs = []
+        for rcg in self.recording_channel_groups():
+            for rc in rcg.recordingchannels:
+                if not api.config.duplicate_channels and rc in all_rcs:
+                    continue
+                all_rcs.append(rc)
+        self.ensure_not_filtered(
+            rc_list, all_rcs,
+            self.parent.get_active_filters('Recording Channel'))
+        self.populate_neo_channel_list()
+
         selection = QItemSelection()
         rcg_set = set(rcg_list)
         for i in self.channel_model.findItems(
@@ -497,6 +566,16 @@ class NeoNavigationDock(QDockWidget, Ui_neoNavigationDock):
             selection, QItemSelectionModel.ClearAndSelect)
 
         # Select units
+        unit_list = [rcg_list[u[1]].units[u[0]]
+                     for u in data['units']]
+        all_units = []
+        for rcg in self.recording_channel_groups():
+            all_units.extend(rcg.units)
+        self.ensure_not_filtered(
+            unit_list, all_units,
+            self.parent.get_active_filters('Unit'))
+        self.populate_neo_unit_list()
+
         selection = QItemSelection()
         for i in self.unit_model.findItems(
                 '*', Qt.MatchWrap | Qt.MatchWildcard):
@@ -510,4 +589,6 @@ class NeoNavigationDock(QDockWidget, Ui_neoNavigationDock):
                 selection.append(QItemSelectionRange(
                     self.unit_model.indexFromItem(i)))
         self.neoUnitList.selectionModel().select(
-            selection, QItemSelectionModel.ClearAndSelect)
\ No newline at end of file
+            selection, QItemSelectionModel.ClearAndSelect)
+
+        self.parent.refresh_filters()
\ No newline at end of file
