@@ -14,8 +14,11 @@ except ImportError:  # Spyder >= 2.2.0beta3
     try:
         from spyderlib.utils.module_completion import module_completion
     except ImportError:  # Spyder >= 2.3.0
-        from spyderlib.utils.introspection.module_completion import \
-            module_completion
+        try:
+            from spyderlib.utils.introspection.module_completion import \
+                module_completion
+        except ImportError:  # Spyder >= 2.3.2
+            from spyderlib.utils.introspection import module_completion
 
 try:
     from spyderlib.utils.dochelpers import getsignaturesfromtext
